import 'package:flutter/material.dart';
import 'package:flutter_slick/navigation/view_model_widget_builder.dart';
import 'item.dart';
import 'paths_builder.dart';
import 'route_path.dart';

typedef RouteSegments = Map<String, String?>;

typedef RouteParameters = Map<String, List<String>>;

class ParsingException implements Exception {
  ParsingException();
}

class RouteArguments {
  RouteArguments();

  factory RouteArguments.parse(
    RouteSegments segments,
    RouteParameters parameters,
  ) {
    return RouteArguments();
  }
}

class RouteBuilder<T extends RouteArguments> {
  final PathsBuilder Function() getBuilder;
  final RegExp pattern;
  final T Function(RouteSegments, RouteParameters) parse;
  final String Function(T) path;
  final Future<Widget> Function(BuildContext, T)? build;
  final ViewModelWidgetBuilder? screenBuilder;
  final Future<RoutePath?> Function(BuildContext, T) rewrite;
  final RoutePath Function(T)? previousPath;
  final List<NavigationItem> Function()? menuItems;
  final List<NavigationItem> Function()? tabItems;

  RouteBuilder({
    required this.getBuilder,
    required this.pattern,
    required this.parse,
    required this.path,
    this.screenBuilder,
    this.build,
    required this.rewrite,
    this.previousPath,
    this.menuItems,
    this.tabItems,
  }) : assert(((build != null && screenBuilder == null) ||
            (screenBuilder != null && build == null)));

  RoutePath<T> call([
    RouteSegments segments = const {},
    RouteParameters parameters = const {},
  ]) =>
      RoutePath<T>(
        this,
        segments,
        parameters,
      );
}
