import 'dart:core';

import 'package:flutter/material.dart';
import 'package:flutter/widgets.dart';
import 'package:provider/provider.dart';

import 'item.dart';
import 'route_builder.dart';

class RoutePath<T extends RouteArguments> {
  final RouteBuilder<T> routeBuilder;
  final RouteSegments segments;
  final RouteParameters parameters;

  RoutePath(
    this.routeBuilder,
    this.segments,
    this.parameters,
  );

  RegExp get pattern => routeBuilder.pattern;

  String get path {
    var arguments = routeBuilder.parse(segments, parameters);
    var result = routeBuilder.getBuilder().path(routeBuilder.path(arguments));
    if (parameters.isNotEmpty) {
      result = result + Uri(queryParameters: parameters).toString();
    }
    return result;
  }

  RoutePath? get previousPath {
    var arguments = routeBuilder.parse(segments, parameters);
    var previousPath = routeBuilder.previousPath;
    return previousPath == null ? null : previousPath(arguments);
  }

  List<NavigationItem>? get menuItems {
    var menuItems = routeBuilder.menuItems;
    return menuItems == null ? null : menuItems();
  }

  List<NavigationItem>? get tabItems {
    var tabItems = routeBuilder.tabItems;
    return tabItems == null ? null : tabItems();
  }

  Future<Widget> build(BuildContext context) {
    var arguments = routeBuilder.parse(segments, parameters);
    if (routeBuilder.build != null) {
      return routeBuilder.build!(context, arguments);
    } else {
      var vm = routeBuilder.screenBuilder!.viewModel;
      return Future.value(
        ChangeNotifierProvider(
          create: (BuildContext context) {
            return vm;
          },
          child: _buildViewModelWidget(context),
        ),
      );
    }
  }

  Widget _buildViewModelWidget(BuildContext context) {
    var vm = routeBuilder.screenBuilder!.viewModel;
    if (vm.loading) {
      return const Text('loading');
    }
    return routeBuilder.screenBuilder!
        .build(context, routeBuilder.screenBuilder!.viewModel);
  }

  Future<RoutePath?> rewrite(BuildContext context) {
    var arguments = routeBuilder.parse(segments, parameters);
    return routeBuilder.rewrite(context, arguments);
  }

  @override
  bool operator ==(Object other) {
    return other is RoutePath && other.path == path;
  }

  @override
  int get hashCode => path.hashCode;
}
